@@ -2,9 +2,9 @@ package Crypt::PK::DH;
 
 use strict;
 use warnings;
-our $VERSION = '0.082_001';
+our $VERSION = '0.056';
 
-require Exporter; our @ISA = qw(Exporter); ### use Exporter 5.57 'import';
+require Exporter; our @ISA = qw(Exporter); ### use Exporter 'import';
 our %EXPORT_TAGS = ( all => [qw( dh_shared_secret )] );
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw();
@@ -168,8 +168,10 @@ my %DH_PARAMS = (
 );
 
 sub new {
-  my $self = shift->_new();
-  return @_ > 0 ? $self->import_key(@_) : $self;
+  my ($class, $f) = @_;
+  my $self = _new();
+  $self->import_key($f) if $f;
+  return  $self;
 }
 
 sub import_key {
@@ -227,9 +229,7 @@ sub generate_key {
   }
   elsif (ref $param eq 'SCALAR') {
     my $data = $$param;
-    if ($data =~ /-----BEGIN DH PARAMETERS-----\s*(.+)\s*-----END DH PARAMETERS-----/s) {
-      $data = pem_to_der($data) or croak "FATAL: PEM/params decode failed";
-    }
+    $data = pem_to_der($data) if $data =~ /-----BEGIN DH PARAMETERS-----\s*(.+)\s*-----END DH PARAMETERS-----/s;
     return $self->_generate_key_dhparam($data);
   }
   elsif (ref $param eq 'HASH') {
