@@ -2,23 +2,24 @@ package Crypt::PK::DSA;
 
 use strict;
 use warnings;
-our $VERSION = '0.082_001';
+our $VERSION = '0.056';
 
-require Exporter; our @ISA = qw(Exporter); ### use Exporter 5.57 'import';
+require Exporter; our @ISA = qw(Exporter); ### use Exporter 'import';
 our %EXPORT_TAGS = ( all => [qw( dsa_encrypt dsa_decrypt dsa_sign_message dsa_verify_message dsa_sign_hash dsa_verify_hash )] );
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw();
 
 use Carp;
-$Carp::Internal{(__PACKAGE__)}++;
 use CryptX;
 use Crypt::Digest 'digest_data';
 use Crypt::Misc qw(read_rawfile encode_b64u decode_b64u encode_b64 decode_b64 pem_to_der der_to_pem);
 use Crypt::PK;
 
 sub new {
-  my $self = shift->_new();
-  return @_ > 0 ? $self->import_key(@_) : $self;
+  my ($class, $f, $p) = @_;
+  my $self = _new();
+  $self->import_key($f, $p) if $f;
+  return  $self;
 }
 
 sub generate_key {
@@ -36,9 +37,7 @@ sub generate_key {
   }
   elsif (@_ == 1 && ref $_[0] eq 'SCALAR') {
     my $data = ${$_[0]};
-    if ($data =~ /-----BEGIN DSA PARAMETERS-----\s*(.+)\s*-----END DSA PARAMETERS-----/s) {
-      $data = pem_to_der($data) or croak "FATAL: PEM/params decode failed";
-    }
+    $data = pem_to_der($data) if $data =~ /-----BEGIN DSA PARAMETERS-----\s*(.+)\s*-----END DSA PARAMETERS-----/s;
     return $self->_generate_key_dsaparam($data);
   }
   croak "FATAL: DSA generate_key - invalid args";
@@ -77,17 +76,14 @@ sub import_key {
   }
   croak "FATAL: invalid key data" unless $data;
 
-  if ($data =~ /-----BEGIN (DSA PRIVATE|DSA PUBLIC|PRIVATE|ENCRYPTED PRIVATE|PUBLIC) KEY-----(.+?)-----END (DSA PRIVATE|DSA PUBLIC|PRIVATE|ENCRYPTED PRIVATE|PUBLIC) KEY-----/s) {
-    return $self->_import_pem($data, $password);
+  if ($data =~ /-----BEGIN (DSA PRIVATE|DSA PUBLIC|PRIVATE|PUBLIC) KEY-----(.*?)-----END/sg) {
+    $data = pem_to_der($data, $password);
+    return $self->_import($data);
   }
-  elsif ($data =~ /-----BEGIN CERTIFICATE-----(.+?)-----END CERTIFICATE-----/s) {
-    return $self->_import_pem($data, undef);
-  }
-  elsif ($data =~ /-----BEGIN OPENSSH PRIVATE KEY-----(.+?)-----END OPENSSH PRIVATE KEY-----/s) {
-    return $self->_import_openssh($data, $password);
-  }
-  elsif ($data =~ /---- BEGIN SSH2 PUBLIC KEY ----(.+?)---- END SSH2 PUBLIC KEY ----/s) {
-    return $self->_import_openssh($data, undef);
+  elsif ($data =~ /---- BEGIN SSH2 PUBLIC KEY ----(.*?)---- END SSH2 PUBLIC KEY ----/sg) {
+    $data = pem_to_der($data);
+    my ($typ, $p, $q, $g, $y) = Crypt::PK::_ssh_parse($data);
+    return $self->_import_hex(unpack('H*',$p), unpack('H*',$q), unpack('H*',$g), undef, unpack('H*',$y)) if $typ && $p && $q && $g && $y && $typ eq 'ssh-dss';
   }
   elsif ($data =~ /ssh-dss\s+(\S+)/) {
     $data = decode_b64("$1");
@@ -95,17 +91,60 @@ sub import_key {
     return $self->_import_hex(unpack('H*',$p), unpack('H*',$q), unpack('H*',$g), undef, unpack('H*',$y)) if $typ && $p && $q && $g && $y && $typ eq 'ssh-dss';
   }
   else {
-    my $rv = eval { $self->_import($data) } || eval { $self->_import_pkcs8($data, $password) };
-    return $rv if $rv;
+    return $self->_import($data);
   }
   croak "FATAL: invalid or unsupported DSA key format";
 }
 
+sub encrypt {
+  my ($self, $data, $hash_name) = @_;
+  $hash_name = Crypt::Digest::_trans_digest_name($hash_name||'SHA1');
+  return $self->_encrypt($data, $hash_name);
+}
+
+sub decrypt {
+  my ($self, $data) = @_;
+  return $self->_decrypt($data);
+}
+
+sub _truncate {
+  my ($self, $hash) = @_;
+  ### section 4.6 of FIPS 186-4
+  # let N be the bit length of q
+  # z = the leftmost min(N, outlen) bits of Hash(M).
+  my $q = $self->size_q; # = size in bytes
+  return $hash if $q >= length($hash);
+  return substr($hash, 0, $q);
+}
+
+sub sign_message {
+  my ($self, $data, $hash_name) = @_;
+  $hash_name ||= 'SHA1';
+  my $data_hash = digest_data($hash_name, $data);
+  return $self->_sign($self->_truncate($data_hash));
+}
+
+sub verify_message {
+  my ($self, $sig, $data, $hash_name) = @_;
+  $hash_name ||= 'SHA1';
+  my $data_hash = digest_data($hash_name, $data);
+  return $self->_verify($sig, $self->_truncate($data_hash));
+}
+
+sub sign_hash {
+  my ($self, $data_hash) = @_;
+  return $self->_sign($self->_truncate($data_hash));
+}
+
+sub verify_hash {
+  my ($self, $sig, $data_hash) = @_;
+  return $self->_verify($sig, $self->_truncate($data_hash));
+}
+
 ### FUNCTIONS
 
 sub dsa_encrypt {
   my $key = shift;
-  local $SIG{__DIE__} = \&CryptX::_croak;
   $key = __PACKAGE__->new($key) unless ref $key;
   carp "FATAL: invalid 'key' param" unless ref($key) eq __PACKAGE__;
   return $key->encrypt(@_);
@@ -113,7 +152,6 @@ sub dsa_encrypt {
 
 sub dsa_decrypt {
   my $key = shift;
-  local $SIG{__DIE__} = \&CryptX::_croak;
   $key = __PACKAGE__->new($key) unless ref $key;
   carp "FATAL: invalid 'key' param" unless ref($key) eq __PACKAGE__;
   return $key->decrypt(@_);
@@ -121,7 +159,6 @@ sub dsa_decrypt {
 
 sub dsa_sign_message {
   my $key = shift;
-  local $SIG{__DIE__} = \&CryptX::_croak;
   $key = __PACKAGE__->new($key) unless ref $key;
   carp "FATAL: invalid 'key' param" unless ref($key) eq __PACKAGE__;
   return $key->sign_message(@_);
@@ -129,7 +166,6 @@ sub dsa_sign_message {
 
 sub dsa_verify_message {
   my $key = shift;
-  local $SIG{__DIE__} = \&CryptX::_croak;
   $key = __PACKAGE__->new($key) unless ref $key;
   carp "FATAL: invalid 'key' param" unless ref($key) eq __PACKAGE__;
   return $key->verify_message(@_);
@@ -137,7 +173,6 @@ sub dsa_verify_message {
 
 sub dsa_sign_hash {
   my $key = shift;
-  local $SIG{__DIE__} = \&CryptX::_croak;
   $key = __PACKAGE__->new($key) unless ref $key;
   carp "FATAL: invalid 'key' param" unless ref($key) eq __PACKAGE__;
   return $key->sign_hash(@_);
@@ -145,7 +180,6 @@ sub dsa_sign_hash {
 
 sub dsa_verify_hash {
   my $key = shift;
-  local $SIG{__DIE__} = \&CryptX::_croak;
   $key = __PACKAGE__->new($key) unless ref $key;
   carp "FATAL: invalid 'key' param" unless ref($key) eq __PACKAGE__;
   return $key->verify_hash(@_);
