@@ -2,7 +2,7 @@ package Crypt::PRNG::Sober128;
 
 use strict;
 use warnings;
-our $VERSION = '0.082_001';
+our $VERSION = '0.056';
 
 use base qw(Crypt::PRNG Exporter);
 our %EXPORT_TAGS = ( all => [qw(random_bytes random_bytes_hex random_bytes_b64 random_bytes_b64u random_string random_string_from rand irand)] );
@@ -10,6 +10,7 @@ our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw();
 
 use CryptX;
+use base 'Crypt::PRNG';
 
 {
   ### stolen from Bytes::Random::Secure
