@@ -2,7 +2,10 @@ package CryptX;
 
 use strict;
 use warnings ;
-our $VERSION = '0.082_001';
+our $VERSION = '0.056';
+
+use base qw(Exporter);
+our @EXPORT_OK = qw();
 
 require XSLoader;
 XSLoader::load('CryptX', $VERSION);
@@ -11,7 +14,21 @@ use Carp;
 my $has_json;
 
 BEGIN {
-  $has_json = 1 if eval { require JSON; 1 };
+  if (eval { require Cpanel::JSON::XS }) {
+    Cpanel::JSON::XS->import(qw(encode_json decode_json));
+    $has_json = 1;
+  }
+  elsif (eval { require JSON::XS }) {
+    JSON::XS->import(qw(encode_json decode_json));
+    $has_json = 2;
+  }
+  elsif (eval { require JSON::PP }) {
+    JSON::PP->import(qw(encode_json decode_json));
+    $has_json = 3;
+  }
+  else {
+    $has_json = 0;
+  }
 }
 
 sub _croak {
@@ -24,13 +41,18 @@ sub _croak {
 }
 
 sub _decode_json {
-  croak "FATAL: cannot find JSON module" if !$has_json;
-  return JSON->new->utf8->decode(shift);
+  croak "FATAL: cannot find JSON::PP or JSON::XS or Cpanel::JSON::XS" if !$has_json;
+  decode_json(shift);
 }
 
 sub _encode_json {
-  croak "FATAL: cannot find JSON module" if !$has_json;
-  return JSON->new->utf8->canonical->encode(shift);
+  croak "FATAL: cannot find JSON::PP or JSON::XS or Cpanel::JSON::XS" if !$has_json;
+  my $data = shift;
+  my $rv = encode_json($data); # non-canonical fallback
+  return(eval { Cpanel::JSON::XS->new->canonical->encode($data) } || $rv) if $has_json == 1;
+  return(eval { JSON::XS->new->canonical->encode($data)         } || $rv) if $has_json == 2;
+  return(eval { JSON::PP->new->canonical->encode($data)         } || $rv) if $has_json == 3;
+  return($rv);
 }
 
 1;
@@ -39,11 +61,13 @@ sub _encode_json {
 
 =head1 NAME
 
-CryptX - Cryptographic toolkit
+CryptX - Cryptographic toolkit (self-contained, no external libraries needed)
 
 =head1 DESCRIPTION
 
-Perl modules providing a cryptography based on L<LibTomCrypt|https://github.com/libtom/libtomcrypt> library.
+Cryptography in CryptX is based on L<https://github.com/libtom/libtomcrypt>
+
+Currently available modules:
 
 =over
 
@@ -75,12 +99,11 @@ L<Crypt::Digest::BLAKE2s_128>, L<Crypt::Digest::BLAKE2s_160>, L<Crypt::Digest::B
 L<Crypt::Digest::CHAES>, L<Crypt::Digest::MD2>, L<Crypt::Digest::MD4>, L<Crypt::Digest::MD5>, L<Crypt::Digest::RIPEMD128>, L<Crypt::Digest::RIPEMD160>,
 L<Crypt::Digest::RIPEMD256>, L<Crypt::Digest::RIPEMD320>, L<Crypt::Digest::SHA1>, L<Crypt::Digest::SHA224>, L<Crypt::Digest::SHA256>, L<Crypt::Digest::SHA384>,
 L<Crypt::Digest::SHA512>, L<Crypt::Digest::SHA512_224>, L<Crypt::Digest::SHA512_256>, L<Crypt::Digest::Tiger192>, L<Crypt::Digest::Whirlpool>,
-L<Crypt::Digest::Keccak224>, L<Crypt::Digest::Keccak256>, L<Crypt::Digest::Keccak384>, L<Crypt::Digest::Keccak512>,
 L<Crypt::Digest::SHA3_224>, L<Crypt::Digest::SHA3_256>, L<Crypt::Digest::SHA3_384>, L<Crypt::Digest::SHA3_512>, L<Crypt::Digest::SHAKE>
 
 =item * Checksums
 
-L<Crypt::Checksum::Adler32>, L<Crypt::Checksum::CRC32>
+L<Crypt::Checksum>, L<Crypt::Checksum::Adler32>, L<Crypt::Checksum::CRC32>
 
 =item * Message Authentication Codes
 
@@ -89,11 +112,11 @@ L<Crypt::Mac::Pelican>, L<Crypt::Mac::PMAC>, L<Crypt::Mac::XCBC>, L<Crypt::Mac::
 
 =item * Public key cryptography
 
-L<Crypt::PK::RSA>, L<Crypt::PK::DSA>, L<Crypt::PK::ECC>, L<Crypt::PK::DH>, L<Crypt::PK::Ed25519>, L<Crypt::PK::X25519>
+L<Crypt::PK::RSA>, L<Crypt::PK::DSA>, L<Crypt::PK::ECC>, L<Crypt::PK::DH>
 
-=item * Cryptographically secure random number generators - see L<Crypt::PRNG> and related modules
+=item * Cryptographically secure random number generators
 
-L<Crypt::PRNG::Fortuna>, L<Crypt::PRNG::Yarrow>, L<Crypt::PRNG::RC4>, L<Crypt::PRNG::Sober128>, L<Crypt::PRNG::ChaCha20>
+L<Crypt::PRNG>, L<Crypt::PRNG::Fortuna>, L<Crypt::PRNG::Yarrow>, L<Crypt::PRNG::RC4>, L<Crypt::PRNG::Sober128>, L<Crypt::PRNG::ChaCha20>
 
 =item * Key derivation functions - PBKDF1, PBKDF2 and HKDF
 
@@ -111,6 +134,6 @@ This program is free software; you can redistribute it and/or modify it under th
 
 =head1 COPYRIGHT
 
-Copyright (c) 2013-2024 DCIT, a.s. L<https://www.dcit.cz> / Karel Miko
+Copyright (c) 2013+ DCIT, a.s. L<http://www.dcit.cz> / Karel Miko
 
 =cut
